/*
 *
 * (C) COPYRIGHT 2015, 2017 ARM Limited. All rights reserved.
 *
 * This program is free software and is provided to you under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation, and any use by you of this program is subject to the terms
 * of such GNU licence.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can access it online at
 * http://www.gnu.org/licenses/gpl-2.0.html.
 *
 * SPDX-License-Identifier: GPL-2.0
 *
 */

#include <linux/errno.h>
#include <linux/export.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/string.h>
#include <linux/delay.h>

#include <mach/irqs.h>
#include <asm/delay.h>
#include <mach/io.h>

#include <mali_kbase_config.h>

static struct kbase_platform_config dummy_platform_config;

struct kbase_platform_config *kbase_get_platform_config(void)
{
	return &dummy_platform_config;
}

#define REGS_GPU_MODULE_START (0x18100000)


#define SYS_CLKEN1_reg    0xb8000114
#define SYS_SRST1_reg     0xb8000104
#define SYS_INT_CTRL_SCPU 0xb8000290

#define REGS_GPU_WRAPPER_MODULE_START (0xb810e000)
#define REGS_GPU_WRAPPER_RESET  (0xb810F000)
#define REGS_GPU_WRAPPER_ENDIAN (0xb810E048)
#define REGS_GPU_WRAPPER_INT_EN (0xb810E050)
#define BIT1                    (1<<1)
void rt_suspend_gpu(void)
{
	//bit 0 is write data
    //CLKEN = 0
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x0); 
}

void rt_extern_gpu_reset(void)
{
	//printk("func=%s line=%d \n",__FUNCTION__,__LINE__);
	//rtd_outl(0xb80089C0,0x10003001);
	//rtd_outl(0xb80089C0,0x00003001);
	//dsb();
	//printk("func=%s line=%d \n",__FUNCTION__,__LINE__);
}


void rt_reset_gpu(void)
{
    unsigned int clkStatus = rtd_inl(SYS_CLKEN1_reg) & (0x1<<3);
    static unsigned int doOnce = 0;
    
    if(doOnce)
    {
        if(clkStatus)
        {
            printk(KERN_EMERG "[rtkgpu %s:%d] clkStatus=%x, skip gpu reset process\n",__FUNCTION__,__LINE__, clkStatus);
            return; 
        }
        else
        {
            printk(KERN_EMERG "[rtkgpu %s:%d] clkStatus=%x\n",__FUNCTION__,__LINE__, clkStatus);
        }
	}
    else
    {
        printk(KERN_EMERG "[rtkgpu %s:%d] first booting, clkStatus=%x\n",__FUNCTION__,__LINE__, clkStatus);
    }
    doOnce = 1;
#ifndef CONFIG_REALTEK_FPGA
	//bit 0 is write data
    //CLKEN = 0
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x0); 
	udelay(1000);
/*
	//RSTN = 0
	rtd_outl(SYS_SRST1_reg,  0x1<<3 | 0x0); 
	udelay(1000);
*/
	//RSTN = 1
	rtd_outl(SYS_SRST1_reg,  0x1<<3 | 0x1); 
	udelay(1000);

	//CLKEN = 1
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x1); 
#endif

#ifdef CONFIG_ARCH_RTK2885M
	rtd_outl(REGS_GPU_WRAPPER_RESET,rtd_inl(REGS_GPU_WRAPPER_RESET)&(~BIT1));
    udelay(1000);
#endif
	
	//gw_endian_swap
	rtd_outl(REGS_GPU_WRAPPER_ENDIAN,0x01);//endian

	//rtd_outl(0xb810E030,0x40<<16|0x20);
#if 0
        //disable mc for gpu
	rtd_outl(0xb80089C0,(rtd_inl(0xb80089C0)& ~(0x1)));

	//rtd_outl(0xb80089C0,(rtd_inl(0xb80089C0)& ~(0x1)));
	//all disable MC
	rtd_outl(0xb80089C0,0x0);

    //enable GPU MC sys3 MC
	//rtd_outl(0xb80089C0,0x00003001);

    //enable CPU MC sys3 MC
	//rtd_outl(0xb80089C0,0x00000010);
    
	//sys3 in-order
	//rtd_outl(0xb800890C,rtd_inl(0xb800890C)| (1<<2));
	
	//rtd_outl(0xb8007480,rtd_inl(0xb8007480) & ~(0x10));
#else

	//enable CPU MC sys3 MC
	//rtd_outl(0xb80089C0,0x00000010);

    
#endif	
    //enable GPU interrupt
	rtd_outl(REGS_GPU_WRAPPER_INT_EN,0x0f);//gpu, mmu, job, event irq

#ifndef CONFIG_REALTEK_FPGA
    //Rounting to SCPU
	rtd_outl(SYS_INT_CTRL_SCPU,0x1<<20 | 0x1);
#endif
	
#ifdef CONFIG_ARCH_RTK2885M
    // Release GPU core reset	
    rtd_outl(REGS_GPU_WRAPPER_RESET,0x3);
#else
	// Release GPU core reset	
    rtd_outl(REGS_GPU_WRAPPER_RESET,0x1);
#endif
	
	udelay(1000);
}

int kbase_platform_early_init(void)
{
	printk("func=%s line=%d \n",__FUNCTION__,__LINE__);
	rt_reset_gpu();
	printk("func=%s line=%d \n",__FUNCTION__,__LINE__);
	/* Nothing needed at this stage */
	return 0;
}

#ifndef CONFIG_OF
int kbase_platform_register(void)
{
	return 0;
}

void kbase_platform_unregister(void)
{
}
#endif
